LINEAR          = 0;                                                         
SINE_IN         = 1;
SINE_OUT        = 2;
SINE_IN_OUT     = 3;
QUINT_IN        = 4;
QUINT_OUT       = 5;
QUINT_IN_OUT    = 6;
QUART_IN        = 7;
QUART_OUT       = 8;
QUART_IN_OUT    = 9;
QUAD_IN         = 10;
QUAD_OUT        = 11;
QUAD_IN_OUT     = 12;
EXPO_IN         = 13;
EXPO_OUT        = 14;
EXPO_IN_OUT     = 15;
ELASTIC_IN      = 16;
ELASTIC_OUT     = 17;
ELASTIC_IN_OUT  = 18;
CUBIC_IN        = 19;
CUBIC_OUT       = 20;
CUBIC_IN_OUT    = 21;
CIRC_IN         = 22;
CIRC_OUT        = 23;
CIRC_IN_OUT     = 24;
BOUNCE_IN       = 25;
BOUNCE_OUT      = 26;
BOUNCE_IN_OUT   = 27;
BACK_IN         = 28;
BACK_OUT        = 29;
BACK_IN_OUT     = 30;

NOT_LOOP        = 0;
NOT_DELAY       = 0;
LOOP            = 1;

DEFAULT_TIME    = 300;

FREE            = 1;        
BUSY            = 0;

Flow            = [];

LAST_FLOW       = 0;

AddPlayer = function(){                                                      

   return new function(){      
      
      this.state = FREE;        
      
      this.go = function(in_animation, in_item, in_duration, in_delay, in_loop, in_tween){                

        this.release();
        
         switch(arguments.length){
         
            case 6:

                  this.tween = in_tween;                     
            case 5:

                  this.loop = in_loop;   
               
            case 4:

                  this.delay = in_delay;                       
                              
            case 3:

                  this.duration = in_duration;         
   
            case 2:                          

               if(!(in_item instanceof Array))
                  in_item = [in_item];
                  
               if(!(in_animation instanceof Array))
                  in_animation = [in_animation]; 
                                          
               this.animation = in_animation;   
               this.item      = in_item;
               
               this.state = BUSY;
               
               this.starting(); 
         }
      }           
      
      this.run = function(){
                       
         IR.AddListener(IR.EVENT_WORK, 0, this.running, this);
      }
      
      this.starting = function(){
         
         if(this.delay != 0)
            IR.SetTimeout(this.delay, this.run, this)
         else
            this.run();
      }
      
      this.release = function(){
      
         this.item       = [];
         this.animation  = []; 
         this.extract    = null;
         this.extracted  = null;
         
         this.duration   = DEFAULT_TIME;  
         this.loop       = NOT_LOOP;
         this.tween      = LINEAR;
         
         this.counter    = 0;
         this.change     = 0;
         this.first      = 0;
         this.value      = 0;
         this.step       = 0;  
         this.delay      = NOT_DELAY; 
      }
      
      this.calculate = function(){

         this.value = IR.Tween(this.tween, this.counter, this.first, this.change, this.duration);   
      }

      this.running = function(in_step){
         
         this.step = in_step;
         
         for(var j = 0; j < this.animation.length; j++){
         
            this.extract = this.animation[j];            
            this.extracted = this.extract();

            this.calculate();
            
            for(var i = 0; i < this.item.length; i++)              
               this.extracted(i);
         }

         this.counter += in_step;  
         
         if(this.counter > this.duration)
            this.stoping();  
      }
      
      this.stoping = function(){   
                          
         this.state      = FREE; 
         this.counter    = 0;
         IR.RemoveListener(IR.EVENT_WORK, 0, this.running);  

         if(this.loop)
            this.starting();              
      }       
   }
} 

AddFlow = function(){

   return Flow[Flow.length] = AddPlayer();
};              
    
Animation = function(in_animation, in_item, in_duration, in_loop, in_tween){   
  
   if(LAST_FLOW >= Flow.length)
      LAST_FLOW = 0;
      
   for(var i = LAST_FLOW; i < Flow.length; i++)
         if(Flow[i].state == FREE){
         
            LAST_FLOW = i + 1;
            Flow[i].go(in_animation, in_item, in_duration, in_loop, in_tween);
            return;
         }
         
   AddFlow().go(in_animation, in_item, in_duration, in_loop, in_tween);     
};

MoveHorrizontal = function(in_first, in_change){
                            
   return function(){
      
      this.change = in_change;
      this.first  = in_first;  

      return function(i){
      
         if(this.counter >= this.duration - this.step)
               this.value = this.change + this.first;  

         this.item[i]["X"] = this.value;
      }
   }
}

MoveVertical = function(in_first, in_change){
                            
   return function(){
      
      if(in_change < in_first)
         this.change = -(in_first - in_change);
      else
         this.change = in_change - in_first; 
            
      this.first  = in_first;     
      
      return function(i){
         this.item[i]["Y"] = this.value;
      }
   }  
}

MoveDiagonal = function(in_first, in_change){
                            
   return function(){
      
      this.change = in_change;
      this.first  = in_first;  
   
      return function(i){     
   
         this.item[i]["X"] = this.item[i]["Y"] = this.value;
      }
   }
}

Rotate = function(in_change){  
   
   return function(){
      
      this.change = in_change;
      this.first  = 0;       
    
      return function(i){
      
         this.item[i]["Angle"] = this.value;
      }
   }
}

Show = function(){

   return function(){
   
      this.change = 255;
      this.first  = 0;       
   
      return function(i){
      
         if(this.counter >= this.duration - this.step)
            this.value = 255;  
            
         this.item[i].GetState(0)["Opacity"] = this.value;     
      }
   }
}

Hide = function(){

   return function(){
   
      this.change = -255;
      this.first  = 255;   
      
      return function(i){
      
         if(this.counter >= this.duration - this.step)
            this.value = 0;  
      
         this.item[i].GetState(0)["Opacity"] = this.value;
      }
   }
}

ScaleXY = function(in_first, in_change){
                            
   return function(){
      
      this.change = in_change;
      this.first  = in_first;  
   
      return function(i){  
      
         if(this.counter >= this.duration - this.step)
            this.value = this.change + this.first;  
   
         this.item[i]["ScaleX"] = this.item[i]["ScaleY"] = this.value;
      }
   }
}

ScaleX = function(in_first, in_change){
                            
   return function(){
      
      this.change = in_change;
      this.first  = in_first;  
   
      return function(i){ 
      
         if(this.counter >= this.duration - this.step)
            this.value = this.change + this.first;      
   
         this.item[i]["ScaleX"] = this.value;
      }
   }
}

ScaleY = function(in_first, in_change){
                            
   return function(){
      
      this.change = in_change;
      this.first  = in_first;  
   
      return function(i){   
      
         if(this.counter >= this.duration - this.step)
            this.value = this.change + this.first;   
   
         this.item[i]["ScaleY"] = this.value;
      }
   }
}


Value = function(in_first, in_change){
                            
   return function(){
      
      this.change = in_change;
      this.first  = in_first;  
   
      return function(i){
      
         if(this.counter >= this.duration - this.step)
            this.value = this.change + this.first;  
                              
         this.item[i]["Value"] = this.value;
      }
   }
}

Text = function(in_first, in_change){
                            
   return function(){
      
      this.change = in_change;
      this.first  = in_first;  
   
      return function(i){
      
         if(this.counter >= this.duration - this.step)
            this.value = this.change + this.first;  
                              
         this.item[i]["Text"] = this.value;
      }
   }
}

Value_to_Text = function(in_first, in_change, in_text){
                            
   return function(){
      
      this.change = in_change;
      this.first  = in_first;
      this.property_0   = in_text; 
   
      return function(i){
      
         if(this.counter >= this.duration - this.step)
            this.value = this.property_0;
          else
            this.value = (String.fromCharCode(this.value) + String.fromCharCode(this.value + 1)).toUpperCase();
                              
         this.item[i]["Text"] = this.value;
      }
   }
}
